#!/usr/bin/env python
import os, sys
import time, subprocess

# Set solver paths
# Comment it if you do not run it.
solvers = {}
solvers['CDN'] = './liblinear/train'
solvers['CGD-GS'] = './cgd_logreg/runCGD.m'
solvers['IPM'] = './l1_logreg-0.8.2/src_c/l1_logreg_train'
solvers['Lassplore'] = './lassplore/runLassplore.m'
solvers['GLMNET'] = './runGLMNET.R'

# The best parameter C is selected by 5-fold cross-validation.
# Comment it if you do not run it.
bestC = {}
#bestC[('leu.scale','lrb')] = 2.0
#bestC[('duke.scale','lrb')] = 2.0
bestC[('a9a','lrb')] = 2.0
bestC[('real-sim','lrb')] = 4.0
bestC[('news20.binary','lrb')] = 64.0
#bestC[('rcv1_test.binary','lrb')] = 4.0
#bestC[('train308.scale','lrb')] = 4.0
#bestC[('leisure.scale','lrb')] = 4.0

Radius = {}
Radius[('leu.scale','lrb')] = 10.1980873508
Radius[('duke.scale','lrb')] = 14.3319841001
Radius[('a9a','lrb')] = 50.6226646937
Radius[('real-sim','lrb')] = 9733.54191781
Radius[('news20.binary','lrb')] = 26844.161698
Radius[('rcv1_test.binary','lrb')] = 27746.0394049
Radius[('train308.scale','lrb')] = 21924.9502919
Radius[('leisure.scale','lrb')] = 83094.6849208

# Set runtime limits (in seconds)
runtime = {}
runtime['leu.scale'] = 50
runtime['duke.scale'] = 50
runtime['a9a'] = 500
runtime['real-sim'] = 1000
runtime['news20.binary'] = 20000
runtime['rcv1_test.binary'] = 50000
runtime['train308.scale'] = 5000
runtime['leisure.scale'] = 50000

# Keep the order of all data sets
all_data = ['leu.scale','duke.scale','a9a','real-sim','news20.binary',\
			'rcv1_test.binary','train308.scale','leisure.scale']

def run(subp, rtime):
	start = time.time()
	stop = start + rtime
	while True:
		out = subp.poll()
		if out is not None:
			return out
		if time.time() >= stop:
			print 'Already reach time limit (%s)!!' %rtime
			print 'Ready to KILL the job (%s)!!' %subp.pid
			os.kill(subp.pid, 9)
		time.sleep(3)

def compare(data):
	# Check to run or not
	if not bestC.has_key((data,'lrb')):
		print '---------------------------------------'
		print data, 'Skpped!!'
		print
		return
	# Check if data exist
	train_data = 'data/%s.train' %data
	test_data = 'data/%s.test' %data
	if not os.path.exists(train_data) or\
	   not os.path.exists(test_data):
		print '---------------------------------------'
		print train_data, 'or', test_data, 'do not exist!!'
		print
		return
	if not runtime.has_key(data):
		runtime[data] = 1000

	cmd = 'wc -l %s' %train_data
	fp = os.popen(cmd)
	no_inst = int(fp.readline().strip().split()[0])
	fp.close()

	C = bestC[(data,'lrb')]
	R = Radius[(data,'lrb')]
	lambda1 = 1.0/C
	lambda2 = 1.0/C/no_inst
	print '---------------------------------------'
	print train_data, no_inst
	print C, R, lambda1, lambda2
	print '---------------------------------------'

	#CDN
	if solvers.has_key('CDN'):
		solver = solvers['CDN']
		eps = 16.0
		log = './logLRb/CDN_%s_%s.log' %(data,C)
		tmplog = '%s.tmp' %log
		cmd = '%s -s 7 -c %s -e %s -n 40 -g 1.2 %s %s'\
			%(solver,C,eps,train_data,test_data)
		if os.path.exists(solver) and not os.path.exists(log):
			print cmd
			fobj = open(tmplog, 'w')
			subp = subprocess.Popen(cmd.split(), shell=False, stdout=fobj)
			run(subp, runtime[data]*5)
			fobj.close()
			os.system('mv %s %s' %(tmplog,log))
		else:
			print 'CDN Skipped!!'
	else:
		print 'CDN Skipped!!'

	#CGD-GS
	if solvers.has_key('CGD-GS'):
		solver = solvers['CGD-GS']
		eps = 1e-8
		max_iter = 100000
		log = './logLRb/CGD-GS_%s_%s.log' %(data,C)
		tmplog = '%s.tmp' %log
		arg = '-nodisplay -r \"cd cgd_logreg;runCGD(\'../data/%s\',%s,%s,%s);cd ..;quit;\"'\
			%(data,C,eps,max_iter)
		if os.path.exists(solver) and not os.path.exists(log):
			print 'matlab', arg
			fobj = open(tmplog, 'w')
			subp = subprocess.Popen(['matlab',arg], shell=False, stdout=fobj)
			run(subp, runtime[data])
			fobj.close()
			os.system('mv %s %s' %(tmplog,log))
		else:
			print 'CGD-GS Skipped!!'
	else:
		print 'CGD-GS Skipped!!'

	#IPM
	if solvers.has_key('IPM'):
		solver = solvers['IPM']
		eps = 1e-8
		model = './tmp/IPM_%s_%s.model' %(data,C)
		log = './logLRb/IPM_%s_%s.log' %(data,C)
		tmplog = '%s.tmp' %log
		cmd = '%s -q -t %s %s.ipm.x %s.ipm.y %s %s %s.ipm.x %s.ipm.y'\
			%(solver,eps,train_data,train_data,lambda2,model,test_data,test_data)
		if os.path.exists(solver) and not os.path.exists(log):
			print cmd
			os.system('%s > %s' %(cmd,tmplog))
#			fobj = open(tmplog, 'w')
#			subp = subprocess.Popen(cmd.split(), shell=False, stdout=fobj)
#			run(subp, runtime[data])
#			fobj.close()
			os.system('mv %s %s' %(tmplog,log))
		else:
			print 'IPM Skipped!!'
	else:
		print 'IPM Skipped!!'

	#Lassplore
	if solvers.has_key('Lassplore'):
		solver = solvers['Lassplore']
		log = './logLRb/Lassplore_%s_%s.log' %(data,C)
		tmplog = '%s.tmp' %log
		arg = '-nodisplay -r \"cd lassplore;runLassplore(\'../data/%s\',%s,%s,\'../tmp/Lassplore_%s_%s.model\');cd ..;quit;\"'\
			%(data,C,R,data,C)
		if os.path.exists(solver) and not os.path.exists(log):
			print 'matlab', arg
			fobj = open(tmplog, 'w')
			subp = subprocess.Popen(['matlab',arg], shell=False, stdout=fobj)
			run(subp, runtime[data])
			fobj.close()
			os.system('mv %s %s' %(tmplog,log))
		else:
			print 'Lassplore Skipped!!'
	else:
		print 'Lassplore Skipped!!'

	#GLMNET
	if solvers.has_key('GLMNET'):
		solver = solvers['GLMNET']
		eps = 16.0
		(loop, g, n) = ('TRUE', 1.3, 45)
		max_iter = 1000
		log = './logLRb/GLMNET_%s_%s.log' %(data,C)
		tmplog = '%s.tmp' %log
		Rmsg = './tmp/GLMNET_%s_%s.Rmsg' %(data,C)
		arg = "--vanilla --slave --args C=%s eps=%s eps_loop=%s eps_g=%s eps_n=%s iter=%s data=\"data/%s\""\
			%(C,eps,loop,g,n,max_iter,data)
		if os.path.exists(solver) and not os.path.exists(log):
			print 'R', arg, '<', solver
			fin = open(solver, 'r')
			fout = open(tmplog, 'w')
			subp = subprocess.Popen(['R',arg],shell=False, stdin=fin,stdout=fout)
			run(subp, runtime[data])
			fin.close()
			fout.close()
			os.system('mv %s %s' %(tmplog,log))
		else:
			print 'GLMNET Skipped!!'
	else:
		print 'GLMNET Skipped!!'


if __name__ == '__main__':
	data_list = []
	for data in all_data:
		if (data,'lrb') in bestC.keys():
			data_list.append(data)
	for data in data_list:
		compare(data)

